%
% Perform Armijo Line Search
%
function lambda = ArmijoLineSearch( fun, Dfun, varargin )

  lambdaMin = 1E-10 ;
  lambdaMax = 1E+10 ;
  c1        = 1E-4  ;
  dumpMin   = 2     ;
  dumpMax   = 100   ;
  
  f0        = feval( fun, 0 ) ;
  Df0       = feval( Dfun, 0 ) ;
  lambda    = 1 ;
  fLambda   = feval( fun, lambda ) ;
  
  if nargin >= 3 
    fref0 = varargin{1} ;
  else
    fref0 = f0 ;
  end

  if fLambda <= fref0 + (lambda*c1) * Df0
    % forward search
    while fLambda <= fref0 + (lambda*c1) * Df0
      if lambda > lambdaMax
        disp( 'ArmijoLineSearch forward search failed!' );
        return ; 
      end
      lambda  = lambda*2 ;
      fLambda = feval( fun, lambda ) ;
    end ;
    lambda = lambda/2 ;
  else
    % backward search
    lambdap = lambda ;
    while fLambda > fref0 + (lambda*c1) * Df0
      if lambda < lambdaMin
        disp( 'ArmijoLineSearch backward search failed!' );
        return ; 
      end
      if lambda > lambdap
        lambdaTmp = cubic( f0, Df0, fLambda, lambda, fLambdap, lambdap ) ;
      else
        lambdaTmp = quadratic( f0, Df0, fLambda, lambda ) ;
      end ;
      lambdap  = lambda ;
      fLambdap = fLambda ;
      lambda   = max( min( lambdaTmp, lambda/dumpMin ), lambda/dumpMax ) ;
      fLambda  = feval( fun, lambda ) ;
    end ;
  end ;
end
%                         _           _   _      
%    __ _ _   _  __ _  __| |_ __ __ _| |_(_) ___ 
%   / _` | | | |/ _` |/ _` | '__/ _` | __| |/ __|
%  | (_| | |_| | (_| | (_| | | | (_| | |_| | (__ 
%   \__, |\__,_|\__,_|\__,_|_|  \__,_|\__|_|\___|
%      |_|                                       
%      
function alpha = quadratic( f0, Df0, fp, p )
  alpha = Df0 * p^2 / ( 2*(f0+Df0*p-fp) ) ;
end
%              _     _      
%    ___ _   _| |__ (_) ___ 
%   / __| | | | '_ \| |/ __|
%  | (__| |_| | |_) | | (__ 
%   \___|\__,_|_.__/|_|\___|
% 
function alpha = cubic( f0, Df0, fl, l, fp, p )
  % f0 + Df0 * x + a * x^2 + b * x^3
  bf1 = fl - (f0+l*Df0) ;
  bf2 = fp - (f0+p*Df0) ;
  bf3 = (p*l)^2*(p-l) ;
  a   = (p^3 * bf1 - l^3 * bf2) / bf3 ;
  b   = (l^2 * bf2 - p^2 * bf1) / bf3 ;

  if abs(b) < 1E-10
    % points disposed near a parabola
    alpha = -Df0 / (2*a) ;
  else
    delta = a^2 - 3*b*Df0 ;
    if delta < 0
      % complex root, using quadratic again
      alpha = quadratic( f0, Df0, fl, l ) ;
    else
      % regular case, using cubic
      alpha = (sqrt(delta)-a)/(3*b) ;
    end;
  end;
end
