%
% Perform Simplified Armijo Line Search
%
function lambda = ArmijoSimpleSearch( fun, Dfun, varargin )

  lambdaMin = 1E-8 ;
  lambdaMax = 1E+8 ;
  tau       = 0.75 ;
  c1        = 1E-3 ;
  lambda    = 1 ;

  if nargin >= 3 
    f0 = varargin{1} ;
  else
    f0 = feval( fun, 0 ) ;
  end

  fLambda = feval( fun,  lambda ) ;
  Df0     = feval( Dfun, 0 ) ;

  if fLambda <= f0 + (lambda*c1) * Df0
    % forward search
    while (lambda < lambdaMax) & (fLambda <= f0 + (lambda*c1) * Df0)
      lambda  = lambda/tau ;
      fLambda = feval(fun, lambda ) ;
    end ;
    lambda = lambda*tau ;
  else
    % backward search
    while (lambda > lambdaMin) & (fLambda > f0 + (lambda*c1) * Df0)
      lambda  = lambda * tau ;
      fLambda = feval( fun, lambda ) ;
    end ;
  end ;
end
