%
% Broyden method for non linear system
%
function varargout = Broyden( F, x0, epsi )

  x  = x0 ;
  Fx = feval( F, x ) ;
  A  = diag(1E-6+abs(Fx)) ;

  % if required save the computed iteration
  if nargout == 2
    varargout{1} = [] ;
    varargout{2} = [] ;
  end ;

  for k=1:1000

    disp(k)

    % if required save the computed iteration
    if nargout == 2
      varargout{1} = [varargout{1}, norm(feval( F, x ),inf)]  ;
      varargout{2} = [varargout{2}, x ] ;
    end ;

    % check for bad termination
    rc = rcond(A) ;
    if ~ isfinite(rc) | rc < 1E-10
      disp('Broyden: bad termination');
      return ;
    end ;

    % compute step size
    s  = -A \ Fx ;

    % check for termination
    if norm( s, inf ) < epsi
      break ;
    end ;

    % update point
    x  = x + s ;

    % update matrix
    Fx = feval( F,  x ) ;
    A  = A + Fx * (s' / dot(s,s)) ;

  end ;

  if nargout == 1
    varargout{1} = x ;
  end ;

end
